@@ -13,9 +13,19 @@
 #define CANONICALIZE_H
 
 #include "c.h"	/* for PATH_MAX */
+#include "strutils.h"
 
 extern char *canonicalize_path(const char *path);
 extern char *canonicalize_path_restricted(const char *path);
 extern char *canonicalize_dm_name(const char *ptname);
 
+extern char *absolute_path(const char *path);
+
+static inline int is_relative_path(const char *path)
+{
+	if (!path || *path == '/')
+		return 0;
+	return 1;
+}
+
 #endif /* CANONICALIZE_H */
