@@ -13,6 +13,7 @@
 
 #include "c.h"
 #include "list.h"
+#include "strutils.h"
 #include "color-names.h"
 #include "debug.h"
 
@@ -52,6 +53,8 @@ struct libscols_symbols {
 	char	*branch;
 	char	*vert;
 	char	*right;
+	char	*title_padding;
+	char	*cell_padding;
 };
 
 /*
@@ -61,6 +64,7 @@ struct libscols_cell {
 	char	*data;
 	char	*color;
 	void    *userdata;
+	int	flags;
 };
 
 
@@ -75,19 +79,34 @@ struct libscols_column {
 	size_t	width_min;	/* minimal width (usually header width) */
 	size_t  width_max;	/* maximal width */
 	size_t  width_avg;	/* average width, used to detect extreme fields */
+	size_t	width_treeart;	/* size of the tree ascii art */
 	double	width_hint;	/* hint (N < 1 is in percent of termwidth) */
 
 	int	flags;
 	int	is_extreme;
 	char	*color;		/* default column color */
+	char	*safechars;	/* do not encode this bytes */
+
+	char	*pending_data;
+	size_t	pending_data_sz;
+	char	*pending_data_buf;
 
 	int (*cmpfunc)(struct libscols_cell *,
 		       struct libscols_cell *,
 		       void *);			/* cells comparison function */
 	void *cmpfunc_data;
 
+	size_t (*wrap_chunksize)(const struct libscols_column *,
+			const char *, void *);
+	char *(*wrap_nextchunk)(const struct libscols_column *,
+			char *, void *);
+	void *wrapfunc_data;
+
+
 	struct libscols_cell	header;
 	struct list_head	cl_columns;
+
+	struct libscols_table	*table;
 };
 
 /*
@@ -122,12 +141,13 @@ enum {
  */
 struct libscols_table {
 	int	refcount;
-	char	*name;		/* optional table table */
+	char	*name;		/* optional table name (for JSON) */
 	size_t	ncols;		/* number of columns */
 	size_t  ntreecols;	/* number of columns with SCOLS_FL_TREE */
 	size_t	nlines;		/* number of lines */
 	size_t	termwidth;	/* terminal width */
 	size_t  termreduce;	/* extra blank space */
+	int	termforce;	/* SCOLS_TERMFORCE_* */
 	FILE	*out;		/* output stream */
 
 	char	*colsep;	/* column separator */
@@ -136,6 +156,7 @@ struct libscols_table {
 	struct list_head	tb_columns;
 	struct list_head	tb_lines;
 	struct libscols_symbols	*symbols;
+	struct libscols_cell	title;		/* optional table title (for humans) */
 
 	int	indent;		/* indention counter */
 	int	indent_last_sep;/* last printed has been line separator */
@@ -145,8 +166,13 @@ struct libscols_table {
 	unsigned int	ascii		:1,	/* don't use unicode */
 			colors_wanted	:1,	/* enable colors */
 			is_term		:1,	/* isatty() */
-			maxout		:1,	/* maximalize output */
-			no_headings	:1;	/* don't print header */
+			padding_debug	:1,	/* output visible padding chars */
+			maxout		:1,	/* maximize output */
+			header_printed  :1,	/* header already printed */
+			priv_symbols	:1,	/* default private symbols */
+			no_headings	:1,	/* don't print header */
+			no_linesep	:1,	/* don't print line separator */
+			no_wrap		:1;	/* never wrap lines */
 };
 
 #define IS_ITER_FORWARD(_i)	((_i)->direction == SCOLS_ITER_FORWARD)
@@ -167,7 +193,7 @@ struct libscols_table {
 	} while(0)
 
 
-static inline int scols_iter_is_last(struct libscols_iter *itr)
+static inline int scols_iter_is_last(const struct libscols_iter *itr)
 {
 	if (!itr || !itr->head || !itr->p)
 		return 0;
