@@ -37,7 +37,7 @@
  * Wed Jun 22 21:41:56 1994, faith@cs.unc.edu:
  *      Added fix from Mike Grupenhoff (kashmir@umiacs.umd.edu)
  * Mon Jul  1 17:01:39 MET DST 1996, janl@math.uio.no:
- *      - Added fix from David.Chapell@mail.trincoll.edu enabeling daemons
+ *      - Added fix from David.Chapell@mail.trincoll.edu enabling daemons
  *	  to use write.
  *      - ANSIed it since I was working on it anyway.
  * 1999-02-22 Arkadiusz Miśkiewicz <misiek@pld.ORG.PL>
@@ -45,37 +45,41 @@
  *
  */
 
-#include <stdio.h>
-#include <unistd.h>
-#include <utmp.h>
 #include <errno.h>
-#include <time.h>
+#include <getopt.h>
+#include <paths.h>
 #include <pwd.h>
-#include <string.h>
-#include <stdlib.h>
 #include <signal.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
 #include <sys/param.h>
 #include <sys/stat.h>
-#include <paths.h>
-#include <getopt.h>
+#include <time.h>
+#include <unistd.h>
+#include <utmp.h>
 
 #include "c.h"
 #include "carefulputc.h"
 #include "closestream.h"
 #include "nls.h"
+#include "strutils.h"
+#include "ttyutils.h"
 #include "xalloc.h"
 
-static void __attribute__ ((__noreturn__)) usage(FILE * out);
-void search_utmp(char *, char *, char *, uid_t);
-void do_write(char *, char *, uid_t);
-void wr_fputs(char *);
-static void __attribute__ ((__noreturn__)) done(int);
-int term_chk(char *, int *, time_t *, int);
-int utmp_chk(char *, char *);
+static sig_atomic_t signal_received = 0;
 
-static gid_t root_access;
+struct write_control {
+	uid_t src_uid;
+	const char *src_login;
+	const char *src_tty_path;
+	const char *src_tty_name;
+	const char *dst_login;
+	char *dst_tty_path;
+	const char *dst_tty_name;
+};
 
-static void __attribute__ ((__noreturn__)) usage(FILE * out)
+static void __attribute__((__noreturn__)) usage(FILE *out)
 {
 	fputs(USAGE_HEADER, out);
 	fprintf(out,
@@ -93,12 +97,208 @@ static void __attribute__ ((__noreturn__)) usage(FILE * out)
 	exit(out == stderr ? EXIT_FAILURE : EXIT_SUCCESS);
 }
 
+/*
+ * check_tty - check that a terminal exists, and get the message bit
+ *     and the access time
+ */
+static int check_tty(const char *tty, int *tty_writeable, time_t *tty_atime, int showerror)
+{
+	struct stat s;
+
+	if (stat(tty, &s) < 0) {
+		if (showerror)
+			warn("%s", tty);
+		return 1;
+	}
+	if (getuid() == 0)	/* root can always write */
+		*tty_writeable = 1;
+	else {
+		if (getegid() != s.st_gid) {
+			warnx(_("effective gid does not match group of %s"), tty);
+			return 1;
+		}
+		*tty_writeable = s.st_mode & S_IWGRP;
+	}
+	if (tty_atime)
+		*tty_atime = s.st_atime;
+	return 0;
+}
+
+/*
+ * check_utmp - checks that the given user is actually logged in on
+ *     the given tty
+ */
+static int check_utmp(const struct write_control *ctl)
+{
+	struct utmp *u;
+	int res = 1;
+
+	utmpname(_PATH_UTMP);
+	setutent();
+
+	while ((u = getutent())) {
+		if (strncmp(ctl->dst_login, u->ut_user, sizeof(u->ut_user)) == 0 &&
+		    strncmp(ctl->dst_tty_name, u->ut_line, sizeof(u->ut_line)) == 0) {
+			res = 0;
+			break;
+		}
+	}
+
+	endutent();
+	return res;
+}
+
+/*
+ * search_utmp - search utmp for the "best" terminal to write to
+ *
+ * Ignores terminals with messages disabled, and of the rest, returns
+ * the one with the most recent access time.  Returns as value the number
+ * of the user's terminals with messages enabled, or -1 if the user is
+ * not logged in at all.
+ *
+ * Special case for writing to yourself - ignore the terminal you're
+ * writing from, unless that's the only terminal with messages enabled.
+ */
+static void search_utmp(struct write_control *ctl)
+{
+	struct utmp *u;
+	time_t best_atime = 0, tty_atime;
+	int num_ttys = 0, valid_ttys = 0, tty_writeable = 0, user_is_me = 0;
+	char path[UT_LINESIZE + 6];
+
+	utmpname(_PATH_UTMP);
+	setutent();
+
+	while ((u = getutent())) {
+		if (strncmp(ctl->dst_login, u->ut_user, sizeof(u->ut_user)) != 0)
+			continue;
+		num_ttys++;
+		sprintf(path, "/dev/%s", u->ut_line);
+		if (check_tty(path, &tty_writeable, &tty_atime, 0))
+			/* bad term? skip */
+			continue;
+		if (ctl->src_uid && !tty_writeable)
+			/* skip ttys with msgs off */
+			continue;
+		if (strcmp(u->ut_line, ctl->src_tty_name) == 0) {
+			user_is_me = 1;
+			/* don't write to yourself */
+			continue;
+		}
+		if (u->ut_type != USER_PROCESS)
+			/* it's not a valid entry */
+			continue;
+		valid_ttys++;
+		if (best_atime < tty_atime) {
+			best_atime = tty_atime;
+			free(ctl->dst_tty_path);
+			ctl->dst_tty_path = xstrdup(path);
+			ctl->dst_tty_name = ctl->dst_tty_path + 5;
+		}
+	}
+
+	endutent();
+	if (num_ttys == 0)
+		errx(EXIT_FAILURE, _("%s is not logged in"), ctl->dst_login);
+	if (valid_ttys == 0) {
+		if (user_is_me) {
+			/* ok, so write to yourself! */
+			if (!ctl->src_tty_path)
+				errx(EXIT_FAILURE, _("can't find your tty's name"));
+			ctl->dst_tty_path = xstrdup(ctl->src_tty_path);
+			ctl->dst_tty_name = ctl->dst_tty_path + 5;
+			return;
+		}
+		errx(EXIT_FAILURE, _("%s has messages disabled"), ctl->dst_login);
+	}
+	if (1 < valid_ttys)
+		warnx(_("%s is logged in more than once; writing to %s"),
+		      ctl->dst_login, ctl->dst_tty_name);
+}
+
+/*
+ * signal_handler - cause write loop to exit
+ */
+static void signal_handler(int signo)
+{
+	signal_received = signo;
+}
+
+/*
+ * write_line - like fputs(), but makes control characters visible and
+ *     turns \n into \r\n.
+ */
+static void write_line(char *s)
+{
+	while (*s) {
+		const int c = *s++;
+
+		if ((c == '\n' && fputc_careful('\r', stdout, '^') == EOF)
+		    || fputc_careful(c, stdout, '^') == EOF)
+			err(EXIT_FAILURE, _("carefulputc failed"));
+	}
+}
+
+/*
+ * do_write - actually make the connection
+ */
+static void do_write(const struct write_control *ctl)
+{
+	char *login, *pwuid;
+	struct passwd *pwd;
+	time_t now;
+	struct tm *tm;
+	char *host, line[512];
+	struct sigaction sigact;
+
+	/* Determine our login name(s) before the we reopen() stdout */
+	if ((pwd = getpwuid(ctl->src_uid)) != NULL)
+		pwuid = pwd->pw_name;
+	else
+		pwuid = "???";
+	if ((login = getlogin()) == NULL)
+		login = pwuid;
+
+	if ((freopen(ctl->dst_tty_path, "w", stdout)) == NULL)
+		err(EXIT_FAILURE, "%s", ctl->dst_tty_path);
+
+	sigact.sa_handler = signal_handler;
+	sigemptyset(&sigact.sa_mask);
+	sigact.sa_flags = 0;
+	sigaction(SIGINT, &sigact, NULL);
+	sigaction(SIGHUP, &sigact, NULL);
+
+	host = xgethostname();
+	if (!host)
+		host = xstrdup("???");
+
+	now = time((time_t *)NULL);
+	tm = localtime(&now);
+	/* print greeting */
+	printf("\r\n\a\a\a");
+	if (strcmp(login, pwuid))
+		printf(_("Message from %s@%s (as %s) on %s at %02d:%02d ..."),
+		       login, host, pwuid, ctl->src_tty_name,
+		       tm->tm_hour, tm->tm_min);
+	else
+		printf(_("Message from %s@%s on %s at %02d:%02d ..."),
+		       login, host, ctl->src_tty_name,
+		       tm->tm_hour, tm->tm_min);
+	free(host);
+	printf("\r\n");
+
+	while (fgets(line, sizeof(line), stdin) != NULL) {
+		if (signal_received)
+			break;
+		write_line(line);
+	}
+	printf("EOF\r\n");
+}
+
 int main(int argc, char **argv)
 {
-	time_t atime;
-	uid_t myuid;
-	int msgsok, myttyfd, c;
-	char tty[PATH_MAX], *mytty;
+	int tty_writeable = 0, c;
+	struct write_control ctl = { 0 };
 
 	static const struct option longopts[] = {
 		{"version", no_argument, NULL, 'V'},
@@ -122,267 +322,48 @@ int main(int argc, char **argv)
 			usage(stderr);
 		}
 
-	root_access = !getegid();
-
-	/* check that sender has write enabled */
-	if (isatty(fileno(stdin)))
-		myttyfd = fileno(stdin);
-	else if (isatty(fileno(stdout)))
-		myttyfd = fileno(stdout);
-	else if (isatty(fileno(stderr)))
-		myttyfd = fileno(stderr);
-	else
-		myttyfd = -1;
-
-	if (myttyfd != -1) {
-		if (!(mytty = ttyname(myttyfd)))
-			errx(EXIT_FAILURE,
-			     _("can't find your tty's name"));
-
-		/*
-		 * We may have /dev/ttyN but also /dev/pts/xx. Below,
-		 * term_chk() will put "/dev/" in front, so remove that
-		 * part.
-		 */
-		if (!strncmp(mytty, "/dev/", 5))
-			mytty += 5;
-		if (term_chk(mytty, &msgsok, &atime, 1))
+	if (get_terminal_name(&ctl.src_tty_path, &ctl.src_tty_name, NULL) == 0) {
+		/* check that sender has write enabled */
+		if (check_tty(ctl.src_tty_path, &tty_writeable, NULL, 1))
 			exit(EXIT_FAILURE);
-		if (!msgsok)
+		if (!tty_writeable)
 			errx(EXIT_FAILURE,
 			     _("you have write permission turned off"));
-
+		tty_writeable = 0;
 	} else
-		mytty = "<no tty>";
+		ctl.src_tty_name = "<no tty>";
 
-	myuid = getuid();
+	ctl.src_uid = getuid();
 
 	/* check args */
 	switch (argc) {
 	case 2:
-		search_utmp(argv[1], tty, mytty, myuid);
-		do_write(tty, mytty, myuid);
+		ctl.dst_login = argv[1];
+		search_utmp(&ctl);
+		do_write(&ctl);
 		break;
 	case 3:
+		ctl.dst_login = argv[1];
 		if (!strncmp(argv[2], "/dev/", 5))
-			argv[2] += 5;
-		if (utmp_chk(argv[1], argv[2]))
+			ctl.dst_tty_path = xstrdup(argv[2]);
+		else
+			xasprintf(&ctl.dst_tty_path, "/dev/%s", argv[2]);
+		ctl.dst_tty_name = ctl.dst_tty_path + 5;
+		if (check_utmp(&ctl))
 			errx(EXIT_FAILURE,
 			     _("%s is not logged in on %s"),
-			     argv[1], argv[2]);
-		if (term_chk(argv[2], &msgsok, &atime, 1))
+			     ctl.dst_login, ctl.dst_tty_name);
+		if (check_tty(ctl.dst_tty_path, &tty_writeable, NULL, 1))
 			exit(EXIT_FAILURE);
-		if (myuid && !msgsok)
+		if (ctl.src_uid && !tty_writeable)
 			errx(EXIT_FAILURE,
 			     _("%s has messages disabled on %s"),
-			     argv[1], argv[2]);
-		do_write(argv[2], mytty, myuid);
+			     ctl.dst_login, ctl.dst_tty_name);
+		do_write(&ctl);
 		break;
 	default:
 		usage(stderr);
 	}
-
-	done(0);
-	/* NOTREACHED */
-	return EXIT_FAILURE;
-}
-
-
-/*
- * utmp_chk - checks that the given user is actually logged in on
- *     the given tty
- */
-int utmp_chk(char *user, char *tty)
-{
-	struct utmp u;
-	struct utmp *uptr;
-	int res = 1;
-
-	utmpname(_PATH_UTMP);
-	setutent();
-
-	while ((uptr = getutent())) {
-		memcpy(&u, uptr, sizeof(u));
-		if (strncmp(user, u.ut_user, sizeof(u.ut_user)) == 0 &&
-		    strncmp(tty, u.ut_line, sizeof(u.ut_line)) == 0) {
-			res = 0;
-			break;
-		}
-	}
-
-	endutent();
-	return res;
-}
-
-/*
- * search_utmp - search utmp for the "best" terminal to write to
- *
- * Ignores terminals with messages disabled, and of the rest, returns
- * the one with the most recent access time.  Returns as value the number
- * of the user's terminals with messages enabled, or -1 if the user is
- * not logged in at all.
- *
- * Special case for writing to yourself - ignore the terminal you're
- * writing from, unless that's the only terminal with messages enabled.
- */
-void search_utmp(char *user, char *tty, char *mytty, uid_t myuid)
-{
-	struct utmp u;
-	struct utmp *uptr;
-	time_t bestatime, atime;
-	int nloggedttys, nttys, msgsok, user_is_me;
-	char atty[sizeof(u.ut_line) + 1];
-
-	utmpname(_PATH_UTMP);
-	setutent();
-
-	nloggedttys = nttys = 0;
-	bestatime = 0;
-	user_is_me = 0;
-	while ((uptr = getutent())) {
-		memcpy(&u, uptr, sizeof(u));
-		if (strncmp(user, u.ut_user, sizeof(u.ut_user)) == 0) {
-			++nloggedttys;
-			strncpy(atty, u.ut_line, sizeof(u.ut_line));
-			atty[sizeof(u.ut_line)] = '\0';
-			if (term_chk(atty, &msgsok, &atime, 0))
-				/* bad term? skip */
-				continue;
-			if (myuid && !msgsok)
-				/* skip ttys with msgs off */
-				continue;
-			if (strcmp(atty, mytty) == 0) {
-				user_is_me = 1;
-				/* don't write to yourself */
-				continue;
-			}
-			if (u.ut_type != USER_PROCESS)
-				/* it's not a valid entry */
-				continue;
-			++nttys;
-			if (atime > bestatime) {
-				bestatime = atime;
-				strcpy(tty, atty);
-			}
-		}
-	}
-
-	endutent();
-	if (nloggedttys == 0)
-		errx(EXIT_FAILURE, _("%s is not logged in"), user);
-	if (nttys == 0) {
-		if (user_is_me) {
-			/* ok, so write to yourself! */
-			strcpy(tty, mytty);
-			return;
-		}
-		errx(EXIT_FAILURE, _("%s has messages disabled"), user);
-	} else if (nttys > 1) {
-		warnx(_("%s is logged in more than once; writing to %s"),
-		      user, tty);
-	}
-}
-
-/*
- * term_chk - check that a terminal exists, and get the message bit
- *     and the access time
- */
-int term_chk(char *tty, int *msgsokP, time_t * atimeP, int showerror)
-{
-	struct stat s;
-	char path[PATH_MAX];
-
-	if (strlen(tty) + 6 > sizeof(path))
-		return 1;
-	sprintf(path, "/dev/%s", tty);
-	if (stat(path, &s) < 0) {
-		if (showerror)
-			warn("%s", path);
-		return 1;
-	}
-
-	*msgsokP = !access(path, W_OK);
-	if (!root_access && *msgsokP)
-		*msgsokP = s.st_mode & S_IWGRP;
-	*atimeP = s.st_atime;
-	return 0;
-}
-
-/*
- * do_write - actually make the connection
- */
-void do_write(char *tty, char *mytty, uid_t myuid)
-{
-	char *login, *pwuid, *nows;
-	struct passwd *pwd;
-	time_t now;
-	char path[PATH_MAX], *host, line[512];
-
-	/* Determine our login name(s) before the we reopen() stdout */
-	if ((pwd = getpwuid(myuid)) != NULL)
-		pwuid = pwd->pw_name;
-	else
-		pwuid = "???";
-	if ((login = getlogin()) == NULL)
-		login = pwuid;
-
-	if (strlen(tty) + 6 > sizeof(path))
-		errx(EXIT_FAILURE, _("tty path %s too long"), tty);
-	snprintf(path, sizeof(path), "/dev/%s", tty);
-	if ((freopen(path, "w", stdout)) == NULL)
-		err(EXIT_FAILURE, "%s", path);
-
-	signal(SIGINT, done);
-	signal(SIGHUP, done);
-
-	/* print greeting */
-	host = xgethostname();
-	if (!host)
-		host = xstrdup("???");
-
-	now = time((time_t *) NULL);
-	nows = ctime(&now);
-	nows[16] = '\0';
-	printf("\r\n\007\007\007");
-	if (strcmp(login, pwuid))
-		printf(_("Message from %s@%s (as %s) on %s at %s ..."),
-		       login, host, pwuid, mytty, nows + 11);
-	else
-		printf(_("Message from %s@%s on %s at %s ..."),
-		       login, host, mytty, nows + 11);
-	free(host);
-	printf("\r\n");
-
-	while (fgets(line, sizeof(line), stdin) != NULL)
-		wr_fputs(line);
-}
-
-/*
- * done - cleanup and exit
- */
-static void __attribute__ ((__noreturn__))
-    done(int dummy __attribute__ ((__unused__)))
-{
-	printf("EOF\r\n");
-	_exit(EXIT_SUCCESS);
-}
-
-/*
- * wr_fputs - like fputs(), but makes control characters visible and
- *     turns \n into \r\n.
- */
-void wr_fputs(char *s)
-{
-	char c;
-
-#define	PUTC(c)	if (fputc_careful(c, stdout, '^') == EOF) \
-    err(EXIT_FAILURE, _("carefulputc failed"));
-	while (*s) {
-		c = *s++;
-		if (c == '\n')
-			PUTC('\r');
-		PUTC(c);
-	}
-	return;
-#undef PUTC
+	free(ctl.dst_tty_path);
+	return EXIT_SUCCESS;
 }
